@@ -2,7 +2,10 @@
 /**
  * Nag_Driver:: defines an API for implementing storage backends for Nag.
  *
- * $Horde: nag/lib/Driver.php,v 1.57.2.4 2005/07/20 01:00:59 chuck Exp $
+ * $Horde: nag/lib/Driver.php,v 1.57.2.14 2006/05/05 16:01:10 jan Exp $
+ *
+ * See the enclosed file COPYING for license information (GPL).  If you
+ * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
  * @since   Nag 0.1
@@ -53,7 +56,7 @@ class Nag_Driver {
     function listAlarms($date)
     {
         if (!count($this->_tasks)) {
-            $ret = $this->retrieve();
+            $ret = $this->retrieve(0);
             if (is_a($ret,'PEAR_Error')) {
                 return $ret;
             }
@@ -111,41 +114,44 @@ class Nag_Driver {
             $vTodo->setAttribute('PRIORITY', $task['priority']);
         }
 
-        if (isset($task['due']) && !empty($task['completed'])) {
+        if (!empty($task['due']) && empty($task['completed'])) {
             $vTodo->setAttribute('DUE', $task['due']);
+            if (!empty($task['alarm'])) {
+                $vTodo->setAttribute('AALARM', $task['due'] - $task['alarm'] * 60);
+            }
         }
 
         if (!empty($task['completed'])) {
             $vTodo->setAttribute('STATUS', 'COMPLETED');
 
-            // Some applications won't consider a task completed
-            // unless it has a COMPLETED: date. Fill in today's date
-            // for now, as we don't currently track completion date in
-            // Nag.
+            /* Some applications won't consider a task completed
+             * unless it has a COMPLETED: date. Fill in today's date
+             * for now, as we don't currently track completion date in
+             * Nag. */
             $vTodo->setAttribute('COMPLETED', time());
+        } else {
+            // status values are defined in rfc2445
+            $vTodo->setAttribute('STATUS', 'NEEDS-ACTION');
         }
 
         if (!empty($task['category'])) {
             $vTodo->setAttribute('CATEGORIES', $task['category']);
         }
 
-        /**
-         * @TODO Alarms.
-         */
-
         /* Get the task's history. */
-        require_once 'Horde/History.php';
         $history = &Horde_History::singleton();
         $log = $history->getHistory('nag:' . $task['tasklist_id'] . ':' . $task['uid']);
-        foreach ($log->getData() as $entry) {
-            switch ($entry['action']) {
-            case 'add':
-                $created = $entry['ts'];
-                break;
+        if ($log && !is_a($log, 'PEAR_Error')) {
+            foreach ($log->getData() as $entry) {
+                switch ($entry['action']) {
+                case 'add':
+                    $created = $entry['ts'];
+                    break;
 
-            case 'modify':
-                $modified = $entry['ts'];
-                break;
+                case 'modify':
+                    $modified = $entry['ts'];
+                    break;
+                }
             }
         }
         if (!empty($created)) {
@@ -179,6 +185,17 @@ class Nag_Driver {
             $r['uid'] = $uid;
         }
 
+        $due = $vTodo->getAttribute('DUE');
+        if (!is_array($due) && !is_a($due, 'PEAR_Error') && !empty($due)) {
+            $r['due'] = $due;
+        }
+
+        $alarm = $vTodo->getAttribute('AALARM');
+        if (!is_array($alarm) && !is_a($alarm, 'PEAR_Error') &&
+            !empty($alarm) && !empty($r['due'])) {
+            $r['alarm'] = intval(($r['due'] - $alarm) / 60);
+        }
+
         $desc = $vTodo->getAttribute('DESCRIPTION');
         if (!is_array($desc) && !is_a($desc, 'PEAR_Error')) {
             $r['desc'] = $desc;
@@ -189,25 +206,20 @@ class Nag_Driver {
             $r['priority'] = $priority;
         }
 
-        $due = $vTodo->getAttribute('DTSTAMP');
-        if (!is_array($due) && !is_a($due, 'PEAR_Error')) {
-            $r['due'] = $due;
-        }
-
         $cat = $vTodo->getAttribute('CATEGORIES');
         if (!is_array($cat) && !is_a($cat, 'PEAR_Error')) {
             $r['category'] = $cat;
         }
 
         $status = $vTodo->getAttribute('STATUS');
-        if (!is_array($status) && !is_a($status, 'PEAR_Error') && !strcasecmp($status, 'COMPLETED')) {
-            $r['completed'] = 1;
+        if (!is_array($status) && !is_a($status, 'PEAR_Error')) {
+            if (!strcasecmp($status, 'COMPLETED')) {
+                $r['completed'] = 1;
+            } else {
+                $r['completed'] = 0;
+            }
         }
 
-        /**
-         * @TODO Alarms.
-         */
-
         return $r;
     }
 
@@ -243,10 +255,12 @@ class Nag_Driver {
         require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
         $class = 'Nag_Driver_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($tasklist, $params);
+            $nag = &new $class($tasklist, $params);
         } else {
-            return false;
+            $nag = false;
         }
+
+        return $nag;
     }
 
     /**
@@ -297,4 +311,112 @@ class Nag_Driver {
         return $instances[$signature];
     }
 
+    /**
+     * Adds a task and handles notification.
+     *
+     * @param string $name        The name (short) of the task.
+     * @param string $desc        The description (long) of the task.
+     * @param integer $due        The due date of the task.
+     * @param integer $priority   The priority of the task.
+     * @param integer $completed  The completion state of the task.
+     * @param string $category    The category of the task.
+     * @param integer $alarm      The alarm associated with the task.
+     * @param string $uid         A Unique Identifier for the task.
+     *
+     * @return array              array(ID,UID) of new task
+     */
+
+    function add($name, $desc, $due = 0, $priority = 0, $completed = 0,
+                 $category = '', $alarm = 0, $uid = null)
+    {
+        if ($uid === null) {
+            $uid = $this->generateUID();
+        }
+
+        $taskId = $this->_add($name, $desc, $due, $priority, $completed,
+                              $category, $alarm, $uid);
+        if (is_a($taskId, 'PEAR_Error')) {
+            return $taskId;
+        }
+
+        /* Log the creation of this item in the history log. */
+        $history = &Horde_History::singleton();
+        $history->log('nag:' . $this->_tasklist . ':' . $uid, array('action' => 'add'), true);
+
+        /* Notify users about the new event. */
+        $result = Nag::sendNotification('add', $this->_tasklist, $name, $desc, $due, $priority);
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
+        return array($taskId, $uid);
+    }
+
+    /**
+     * Modifies an existing task and handles notification.
+     *
+     * @param string $taskId      The task to modify.
+     * @param string $name        The name (short) of the task.
+     * @param string $desc        The description (long) of the task.
+     * @param integer $due        The due date of the task.
+     * @param integer $priority   The priority of the task.
+     * @param integer $completed  The completion state of the task.
+     * @param string $category    The category of the task.
+     * @param integer $alarm      The alarm associated with the task.
+     */
+    function modify($taskId, $name, $desc, $due = 0, $priority = 0,
+                    $completed = 0, $category = '', $alarm = 0)
+    {
+        $modify = $this->_modify($taskId, $name, $desc, $due, $priority,
+                                 $completed, $category, $alarm);
+        if (is_a($modify, 'PEAR_Error')) {
+            return $modify;
+        }
+
+        /* Log the modification of this item in the history log. */
+        $task = $this->get($taskId);
+        if (!empty($task['uid'])) {
+            $history = &Horde_History::singleton();
+            $history->log('nag:' . $this->_tasklist . ':' . $task['uid'], array('action' => 'modify'), true);
+        }
+
+        /* Notify users about the new event. */
+        $result = Nag::sendNotification('edit', $this->_tasklist, $name, $desc, $due, $priority);
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
+        return true;
+    }
+
+    /**
+     * Deletes a task and handles notification.
+     *
+     * @param string $taskId  The task to delete.
+     */
+    function delete($taskId)
+    {
+        /* Get the task's details for use later. */
+        $task = $this->get($taskId);
+
+        $delete = $this->_delete($taskId);
+        if (is_a($delete, 'PEAR_Error')) {
+            return $delete;
+        }
+
+        /* Log the deletion of this item in the history log. */
+        if (!empty($task['uid'])) {
+            $history = &Horde_History::singleton();
+            $history->log('nag:' . $this->_tasklist . ':' . $task['uid'], array('action' => 'delete'), true);
+        }
+
+        /* Notify users about the new event. */
+        $result = Nag::sendNotification('delete', $this->_tasklist, $task['name'], $task['desc'], $task['due'], $task['priority']);
+        if (is_a($result, 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+        }
+
+        return true;
+    }
+
 }
