@@ -1,4 +1,4 @@
--- $Horde: nag/scripts/upgrades/1.1_to_2.0.mysql.sql,v 1.6 2004/12/21 14:56:54 chuck Exp $
+-- $Horde: nag/scripts/upgrades/1.1_to_2.0.mysql.sql,v 1.6.2.2 2005/10/18 12:56:50 jan Exp $
 --
 -- You can simply execute this file in your database.
 --
@@ -23,5 +23,3 @@ UPDATE nag_tasks SET task_uid = CONCAT('nag:', task_id) WHERE task_id NOT LIKE '
 
 ALTER TABLE nag_tasks DROP PRIMARY KEY;
 ALTER TABLE nag_tasks ADD PRIMARY KEY (task_id);
-
-CREATE INDEX nag_uid_idx ON nag_tasks (task_uid);
